==> baloo-4.12.95.tar.xz: baloo-4.12.95/src/file/commitqueue.cpp <==
/*
 * This file is part of the KDE Baloo Project
 * Copyright (C) 2013  Vishesh Handa <me@vhanda.in>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) version 3, or any
 * later version accepted by the membership of KDE e.V. (or its
 * successor approved by the membership of KDE e.V.), which shall
 * act as a proxy defined in Section 6 of version 3 of the license.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "commitqueue.h"
#include "database.h"

#include <KDebug>
#include <KDiskFreeSpaceInfo>
#include <QCoreApplication>

Baloo::CommitQueue::CommitQueue(Database* db, QObject* parent)
    : QObject(parent)
    , m_db(db)
{
    m_smallTimer.setSingleShot(true);
    m_smallTimer.setInterval(200);
    connect(&m_smallTimer, SIGNAL(timeout()), this, SLOT(commit()));

    m_largeTimer.setSingleShot(true);
    m_largeTimer.setInterval(10000);
    connect(&m_largeTimer, SIGNAL(timeout()), this, SLOT(commit()));
}

Baloo::CommitQueue::~CommitQueue()
{
    commit();
}

void Baloo::CommitQueue::add(unsigned id, Xapian::Document doc)
{
    m_docsToAdd << qMakePair(id, doc);
    startTimers();
}

void Baloo::CommitQueue::remove(unsigned int docid)
{
    m_docsToRemove << docid;
    startTimers();
}

void Baloo::CommitQueue::startTimers()
{
    m_smallTimer.start();
    if (!m_largeTimer.isActive()) {
        m_largeTimer.start();
    }
}


void Baloo::CommitQueue::commit()
{
    // The 200 mb is arbitrary
    KDiskFreeSpaceInfo info = KDiskFreeSpaceInfo::freeSpaceInfo(m_db->path());
    if (info.isValid() && info.available() <= 200 * 1024 * 1024) {
        kError() << "Low disk space. Aborting!!";
        QCoreApplication::instance()->quit();
        return;
    }

    m_db->sqlDatabase().commit();
    m_db->sqlDatabase().transaction();
    kDebug() << "SQL Committed";

    if (m_docsToAdd.isEmpty() && m_docsToRemove.isEmpty())
        return;

    const QByteArray path = m_db->path().toUtf8();
    try {
        Xapian::WritableDatabase db(path.constData(), Xapian::DB_CREATE_OR_OPEN);

        kDebug() << "Adding:" << m_docsToAdd.size() << "docs";
        Q_FOREACH (const DocIdPair& doc, m_docsToAdd) {
            db.replace_document(doc.first, doc.second);
        }
        m_docsToAdd.clear();

        kDebug() << "Removing:" << m_docsToRemove.size() << "docs";
        Q_FOREACH (Xapian::docid id, m_docsToRemove) {
            try {
                db.delete_document(id);
            }
            catch (const Xapian::DocNotFoundError&) {
            }
        }
        m_docsToRemove.clear();

        db.commit();
        kDebug() << "Xapian Committed";
        m_db->xapianDatabase()->reopen();

        Q_EMIT committed();
    }
    catch (const Xapian::DatabaseLockError& err) {
        kError() << err.get_msg().c_str();
        startTimers();
    }
}

==> baloo-4.12.97.tar.xz: baloo-4.12.97/src/file/commitqueue.cpp <==
/*
 * This file is part of the KDE Baloo Project
 * Copyright (C) 2013  Vishesh Handa <me@vhanda.in>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) version 3, or any
 * later version accepted by the membership of KDE e.V. (or its
 * successor approved by the membership of KDE e.V.), which shall
 * act as a proxy defined in Section 6 of version 3 of the license.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "commitqueue.h"
#include "database.h"

#include <KDebug>
#include <KDiskFreeSpaceInfo>
#include <QCoreApplication>

#include "xapiandocument.h"

Baloo::CommitQueue::CommitQueue(Database* db, QObject* parent)
    : QObject(parent)
    , m_db(db)
{
    m_smallTimer.setSingleShot(true);
    m_smallTimer.setInterval(200);
    connect(&m_smallTimer, SIGNAL(timeout()), this, SLOT(commit()));

    m_largeTimer.setSingleShot(true);
    m_largeTimer.setInterval(10000);
    connect(&m_largeTimer, SIGNAL(timeout()), this, SLOT(commit()));

    connect(m_db->xapianDatabase(), SIGNAL(committed()),
            this, SIGNAL(committed()));
}

Baloo::CommitQueue::~CommitQueue()
{
    commit();
}

void Baloo::CommitQueue::add(unsigned id, Xapian::Document doc)
{
    m_db->xapianDatabase()->replaceDocument(id, doc);
    startTimers();
}

void Baloo::CommitQueue::remove(unsigned int docid)
{
    m_db->xapianDatabase()->deleteDocument(docid);
    startTimers();
}

void Baloo::CommitQueue::startTimers()
{
    m_smallTimer.start();
    if (!m_largeTimer.isActive()) {
        m_largeTimer.start();
    }
}


void Baloo::CommitQueue::commit()
{
    // The 200 mb is arbitrary
    KDiskFreeSpaceInfo info = KDiskFreeSpaceInfo::freeSpaceInfo(m_db->path());
    if (info.isValid() && info.available() <= 200 * 1024 * 1024) {
        kError() << "Low disk space. Aborting!!";
        QCoreApplication::instance()->quit();
        return;
    }

    m_db->sqlDatabase().commit();
    m_db->sqlDatabase().transaction();
    kDebug() << "SQL Committed";

    m_db->xapianDatabase()->commit();
}
